@@ -1,39 +1,46 @@
-/* Copyright (C) 2014 J.F.Dockes
- *       This program is free software; you can redistribute it and/or modify
- *       it under the terms of the GNU General Public License as published by
- *       the Free Software Foundation; either version 2 of the License, or
- *       (at your option) any later version.
+/* Copyright (C) 2006-2016 J.F.Dockes
  *
- *       This program is distributed in the hope that it will be useful,
- *       but WITHOUT ANY WARRANTY; without even the implied warranty of
- *       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *       GNU General Public License for more details.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *       You should have received a copy of the GNU General Public License
- *       along with this program; if not, write to the
- *       Free Software Foundation, Inc.,
- *       59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ *   02110-1301 USA
  */
+#include "libupnpp/config.h"
 
 #include "libupnpp/control/avtransport.hxx"
 
-#include <stdlib.h>                     // for atoi
-#include <upnp/upnp.h>                  // for UPNP_E_SUCCESS, etc
+#include <stdlib.h>
+#include <upnp.h>
 
-#include <ostream>                      // for basic_ostream, endl, etc
-#include <string>                       // for string, basic_string, etc
-#include <utility>                      // for pair
-#include <vector>                       // for vector
+#include <ostream>
+#include <string>
+#include <utility>
+#include <vector>
 
-#include "libupnpp/control/avlastchg.hxx"  // for decodeAVLastChange
-#include "libupnpp/control/cdircontent.hxx"  // for UPnPDirContent, etc
-#include "libupnpp/log.hxx"             // for LOGERR, LOGDEB1, LOGDEB, etc
-#include "libupnpp/soaphelp.hxx"        // for SoapIncoming, etc
-#include "libupnpp/upnpavutils.hxx"     // for upnpdurationtos, etc
-#include "libupnpp/upnpp_p.hxx"         // for stringuppercmp, etc
+#include "libupnpp/control/avlastchg.hxx"
+#include "libupnpp/control/cdircontent.hxx"
+#include "libupnpp/log.hxx"
+#include "libupnpp/soaphelp.hxx"
+#include "libupnpp/upnpavutils.hxx"
+#include "libupnpp/upnpp_p.hxx"
+#include "libupnpp/smallut.h" 
+
+#ifdef _WIN32
+#undef DeviceCapabilities
+#endif
 
 using namespace std;
-using namespace STD_PLACEHOLDERS;
+using namespace std::placeholders;
 using namespace UPnPP;
 
 namespace UPnPClient {
@@ -47,6 +54,10 @@ bool AVTransport::isAVTService(const string& st)
     const string::size_type sz(SType.size()-2);
     return !SType.compare(0, sz, st, 0, sz);
 }
+bool AVTransport::serviceTypeMatch(const std::string& tp)
+{
+    return isAVTService(tp);
+}
 
 static AVTransport::TransportState stringToTpState(const string& s)
 {
@@ -65,7 +76,7 @@ static AVTransport::TransportState stringToTpState(const string& s)
     } else if (!stringuppercmp("NO_MEDIA_PRESENT", s)) {
         return AVTransport::NoMediaPresent;
     } else {
-        LOGINF("AVTransport event: bad value for TransportState: " 
+        LOGINF("AVTransport event: bad value for TransportState: "
                << s << endl);
         return AVTransport::Unknown;
     }
@@ -78,7 +89,7 @@ static AVTransport::TransportStatus stringToTpStatus(const string& s)
     } else if (!stringuppercmp("ERROR_OCCURRED", s)) {
         return  AVTransport::TPS_Error;
     } else {
-        LOGERR("AVTransport event: bad value for TransportStatus: " 
+        LOGERR("AVTransport event: bad value for TransportStatus: "
                << s << endl);
         return  AVTransport::TPS_Unknown;
     }
@@ -99,107 +110,102 @@ static AVTransport::PlayMode stringToPlayMode(const string& s)
     } else if (!stringuppercmp("DIRECT_1", s)) {
         return AVTransport::PM_Direct1;
     } else {
-        LOGERR("AVTransport event: bad value for PlayMode: " 
+        LOGERR("AVTransport event: bad value for PlayMode: "
                << s << endl);
         return AVTransport::PM_Unknown;
     }
 }
 
 void AVTransport::evtCallback(
-    const STD_UNORDERED_MAP<std::string, std::string>& props)
+    const std::unordered_map<std::string, std::string>& props)
 {
     LOGDEB1("AVTransport::evtCallback:" << endl);
-    for (STD_UNORDERED_MAP<std::string, std::string>::const_iterator it = 
-             props.begin(); it != props.end(); it++) {
-        if (it->first.compare("LastChange")) {
+    auto reporter = getReporter();
+    if (nullptr == reporter) {
+        LOGDEB1("AVTransport::evtCallback: " << varchg.first << " -> "
+                << varchg.second << endl);
+        return;
+    }
+
+    for (const auto& prop : props) {
+        if (prop.first.compare("LastChange")) {
             LOGINF("AVTransport:event: var not lastchange: "
-                   << it->first << " -> " << it->second << endl;);
+                   << prop.first << " -> " << prop.second << endl);
             continue;
         }
         LOGDEB1("AVTransport:event: "
-                << it->first << " -> " << it->second << endl;);
+                << prop.first << " -> " << prop.second << endl);
 
-        STD_UNORDERED_MAP<std::string, std::string> props1;
-        if (!decodeAVLastChange(it->second, props1)) {
+        std::unordered_map<std::string, std::string> changes;
+        if (!decodeAVLastChange(prop.second, changes)) {
             LOGERR("AVTransport::evtCallback: bad LastChange value: "
-                   << it->second << endl);
+                   << prop.second << endl);
             return;
         }
-        for (STD_UNORDERED_MAP<std::string, std::string>::iterator it1 = 
-                 props1.begin(); it1 != props1.end(); it1++) {
-            if (!getReporter()) {
-                LOGDEB1("AVTransport::evtCallback: " << it1->first << " -> " 
-                       << it1->second << endl);
-                continue;
-            }
+        for (const auto& varchg : changes) {
+            const std::string& varnm{varchg.first};
+            const std::string& varvalue{varchg.second};
 
-            if (!it1->first.compare("TransportState")) {
-                getReporter()->changed(it1->first.c_str(), 
-                                    stringToTpState(it1->second));
+            if (!varnm.compare("TransportState")) {
+                reporter->changed(varnm.c_str(), stringToTpState(varvalue));
 
-            } else if (!it1->first.compare("TransportStatus")) {
-                getReporter()->changed(it1->first.c_str(), 
-                                    stringToTpStatus(it1->second));
+            } else if (!varnm.compare("TransportStatus")) {
+                reporter->changed(varnm.c_str(), stringToTpStatus(varvalue));
 
-            } else if (!it1->first.compare("CurrentPlayMode")) {
-                getReporter()->changed(it1->first.c_str(), 
-                                    stringToPlayMode(it1->second));
+            } else if (!varnm.compare("CurrentPlayMode")) {
+                reporter->changed(varnm.c_str(), stringToPlayMode(varvalue));
 
-            } else if (!it1->first.compare("CurrentTransportActions")) {
+            } else if (!varnm.compare("CurrentTransportActions")) {
                 int iacts;
-                if (!CTAStringToBits(it1->second, iacts))
-                    getReporter()->changed(it1->first.c_str(), iacts);
+                if (!CTAStringToBits(varvalue, iacts))
+                    reporter->changed(varnm.c_str(), iacts);
 
-            } else if (!it1->first.compare("CurrentTrackURI") ||
-                       !it1->first.compare("AVTransportURI") ||
-                       !it1->first.compare("NextAVTransportURI")) {
-                getReporter()->changed(it1->first.c_str(), 
-                                    it1->second.c_str());
+            } else if (!varnm.compare("CurrentTrackURI") ||
+                       !varnm.compare("AVTransportURI") ||
+                       !varnm.compare("NextAVTransportURI")) {
+                reporter->changed(varnm.c_str(), varvalue.c_str());
 
-            } else if (!it1->first.compare("TransportPlaySpeed") ||
-                       !it1->first.compare("CurrentTrack") ||
-                       !it1->first.compare("NumberOfTracks") ||
-                       !it1->first.compare("RelativeCounterPosition") ||
-                       !it1->first.compare("AbsoluteCounterPosition") ||
-                       !it1->first.compare("InstanceID")) {
-                getReporter()->changed(it1->first.c_str(),
-                                    atoi(it1->second.c_str()));
+            } else if (!varnm.compare("TransportPlaySpeed") ||
+                       !varnm.compare("CurrentTrack") ||
+                       !varnm.compare("NumberOfTracks") ||
+                       !varnm.compare("RelativeCounterPosition") ||
+                       !varnm.compare("AbsoluteCounterPosition") ||
+                       !varnm.compare("InstanceID")) {
+                reporter->changed(varnm.c_str(), atoi(varvalue.c_str()));
 
-            } else if (!it1->first.compare("CurrentMediaDuration") ||
-                       !it1->first.compare("CurrentTrackDuration") ||
-                       !it1->first.compare("RelativeTimePosition") ||
-                       !it1->first.compare("AbsoluteTimePosition")) {
-                getReporter()->changed(it1->first.c_str(),
-                                    upnpdurationtos(it1->second));
+            } else if (!varnm.compare("CurrentMediaDuration") ||
+                       !varnm.compare("CurrentTrackDuration") ||
+                       !varnm.compare("RelativeTimePosition") ||
+                       !varnm.compare("AbsoluteTimePosition")) {
+                reporter->changed(varnm.c_str(), upnpdurationtos(varvalue));
 
-            } else if (!it1->first.compare("AVTransportURIMetaData") ||
-                       !it1->first.compare("NextAVTransportURIMetaData") ||
-                       !it1->first.compare("CurrentTrackMetaData")) {
+            } else if (!varnm.compare("AVTransportURIMetaData") ||
+                       !varnm.compare("NextAVTransportURIMetaData") ||
+                       !varnm.compare("CurrentTrackMetaData")) {
                 UPnPDirContent meta;
-                if (!meta.parse(it1->second)) {
+                if (!varvalue.empty() && !meta.parse(varvalue)) {
                     LOGERR("AVTransport event: bad metadata: [" <<
-                           it1->second << "]" << endl);
+                           varvalue << "]" << endl);
                 } else {
                     LOGDEB1("AVTransport event: good metadata: [" <<
-                            it1->second << "]" << endl);
+                            varvalue << "]" << endl);
                     if (meta.m_items.size() > 0) {
-                        getReporter()->changed(it1->first.c_str(), 
-                                            meta.m_items[0]);
+                        reporter->changed(varnm.c_str(), meta.m_items[0]);
                     }
                 }
-            } else if (!it1->first.compare("PlaybackStorageMedium") ||
-                       !it1->first.compare("PossiblePlaybackStorageMedia") ||
-                       !it1->first.compare("RecordStorageMedium") ||
-                       !it1->first.compare("PossibleRecordStorageMedia") ||
-                       !it1->first.compare("RecordMediumWriteStatus") ||
-                       !it1->first.compare("CurrentRecordQualityMode") ||
-                       !it1->first.compare("PossibleRecordQualityModes")){
-                getReporter()->changed(it1->first.c_str(),it1->second.c_str());
+            } else if (!varnm.compare("PlaybackStorageMedium") ||
+                       !varnm.compare("PossiblePlaybackStorageMedia") ||
+                       !varnm.compare("RecordStorageMedium") ||
+                       !varnm.compare("PossibleRecordStorageMedia") ||
+                       !varnm.compare("RecordMediumWriteStatus") ||
+                       !varnm.compare("CurrentRecordQualityMode") ||
+                       !varnm.compare("PossibleRecordQualityModes")) {
+                reporter->changed(varnm.c_str(), varvalue.c_str());
 
             } else {
                 LOGDEB1("AVTransport event: unknown variable: name [" <<
-                        it1->first << "] value [" << it1->second << endl);
-                getReporter()->changed(it1->first.c_str(),it1->second.c_str());
+                        varnm << "] value [" << varvalue << endl);
+                reporter->changed(varnm.c_str(), varvalue.c_str());
             }
         }
     }
@@ -210,10 +216,10 @@ int AVTransport::setURI(const string& uri, const string& metadata,
                         int instanceID, bool next)
 {
     SoapOutgoing args(getServiceType(), next ? "SetNextAVTransportURI" :
-                         "SetAVTransportURI");
+                      "SetAVTransportURI");
     args("InstanceID", SoapHelp::i2s(instanceID))
-        (next ? "NextURI" : "CurrentURI", uri)
-        (next ? "NextURIMetaData" : "CurrentURIMetaData", metadata);
+    (next ? "NextURI" : "CurrentURI", uri)
+    (next ? "NextURIMetaData" : "CurrentURIMetaData", metadata);
 
     SoapIncoming data;
     return runAction(args, data);
@@ -224,17 +230,31 @@ int AVTransport::setPlayMode(PlayMode pm, int instanceID)
     SoapOutgoing args(getServiceType(), "SetPlayMode");
     string playmode;
     switch (pm) {
-    case PM_Normal: playmode = "NORMAL"; break;
-    case PM_Shuffle: playmode = "SHUFFLE"; break;
-    case PM_RepeatOne: playmode = "REPEAT_ONE"; break;
-    case PM_RepeatAll: playmode = "REPEAT_ALL"; break;
-    case PM_Random: playmode = "RANDOM"; break;
-    case PM_Direct1: playmode = "DIRECT_1"; break;
-    default: playmode = "NORMAL"; break;
+    case PM_Normal:
+        playmode = "NORMAL";
+        break;
+    case PM_Shuffle:
+        playmode = "SHUFFLE";
+        break;
+    case PM_RepeatOne:
+        playmode = "REPEAT_ONE";
+        break;
+    case PM_RepeatAll:
+        playmode = "REPEAT_ALL";
+        break;
+    case PM_Random:
+        playmode = "RANDOM";
+        break;
+    case PM_Direct1:
+        playmode = "DIRECT_1";
+        break;
+    default:
+        playmode = "NORMAL";
+        break;
     }
 
     args("InstanceID", SoapHelp::i2s(instanceID))
-        ("NewPlayMode", playmode);
+    ("NewPlayMode", playmode);
 
     SoapIncoming data;
     return runAction(args, data);
@@ -256,7 +276,8 @@ int AVTransport::getMediaInfo(MediaInfo& info, int instanceID)
     data.get("CurrentURI", &info.cururi);
     data.get("CurrentURIMetaData", &s);
     UPnPDirContent meta;
-    meta.parse(s);
+    if (!s.empty())
+        meta.parse(s);
     if (meta.m_items.size() > 0)
         info.curmeta = meta.m_items[0];
     meta.clear();
@@ -302,13 +323,15 @@ int AVTransport::getPositionInfo(PositionInfo& info, int instanceID)
     data.get("TrackDuration", &s);
     info.trackduration = upnpdurationtos(s);
     data.get("TrackMetaData", &s);
-    UPnPDirContent meta;
-    meta.parse(s);
-    if (meta.m_items.size() > 0) {
-        info.trackmeta = meta.m_items[0];
-        LOGDEB1("AVTransport::getPositionInfo: size " << 
-               meta.m_items.size() << " current title: " 
-               << meta.m_items[0].m_title << endl);
+    if (!s.empty()) {
+        UPnPDirContent meta;
+        meta.parse(s);
+        if (meta.m_items.size() > 0) {
+            info.trackmeta = meta.m_items[0];
+            LOGDEB1("AVTransport::getPositionInfo: size " <<
+                    meta.m_items.size() << " current title: "
+                    << meta.m_items[0].m_title << endl);
+        }
     }
     data.get("TrackURI", &info.trackuri);
     data.get("RelTime", &s);
@@ -378,25 +401,28 @@ int AVTransport::CTAStringToBits(const string& actions, int& iacts)
         return UPNP_E_BAD_RESPONSE;
     }
     iacts = 0;
-    for (vector<string>::iterator it = sacts.begin(); it != sacts.end(); it++) {
-        trimstring(*it);
-        if (!it->compare("Next")) {
+    for (auto& act : sacts) {
+        trimstring(act);
+        // It's not clear if the values are case sensitive, they are
+        // as below in the doc. gmediarender for one, uses
+        // all-caps. So let's compare insensitively.
+        if (!stringicmp(act, "Next")) {
             iacts |= TPA_Next;
-        } else if (!it->compare("Pause")) {
+        } else if (!stringicmp(act, "Pause")) {
             iacts |= TPA_Pause;
-        } else if (!it->compare("Play")) {
+        } else if (!stringicmp(act, "Play")) {
             iacts |= TPA_Play;
-        } else if (!it->compare("Previous")) {
+        } else if (!stringicmp(act, "Previous")) {
             iacts |= TPA_Previous;
-        } else if (!it->compare("Seek")) {
+        } else if (!stringicmp(act, "Seek")) {
             iacts |= TPA_Seek;
-        } else if (!it->compare("Stop")) {
+        } else if (!stringicmp(act, "Stop")) {
             iacts |= TPA_Stop;
-        } else if (it->empty()) {
+        } else if (act.empty()) {
             continue;
         } else {
             LOGERR("AVTransport::CTAStringToBits: unknown action in " <<
-                   actions << " : [" << *it << "]" << endl);
+                   actions << " : [" << act << "]" << endl);
         }
     }
     return 0;
@@ -422,7 +448,7 @@ int AVTransport::play(int speed, int instanceID)
 {
     SoapOutgoing args(getServiceType(), "Play");
     args("InstanceID", SoapHelp::i2s(instanceID))
-        ("Speed", SoapHelp::i2s(speed));
+    ("Speed", SoapHelp::i2s(speed));
     SoapIncoming data;
     return runAction(args, data);
 }
@@ -432,22 +458,40 @@ int AVTransport::seek(SeekMode mode, int target, int instanceID)
     string sm;
     string value = SoapHelp::i2s(target);
     switch (mode) {
-    case SEEK_TRACK_NR: sm = "TRACK_NR"; break;
-    case SEEK_ABS_TIME: sm = "ABS_TIME";value = upnpduration(target*1000);break;
-    case SEEK_REL_TIME: sm = "REL_TIME";value = upnpduration(target*1000);break;
-    case SEEK_ABS_COUNT: sm = "ABS_COUNT"; break;
-    case SEEK_REL_COUNT: sm = "REL_COUNT"; break;
-    case SEEK_CHANNEL_FREQ: sm = "CHANNEL_FREQ"; break;
-    case SEEK_TAPE_INDEX: sm = "TAPE-INDEX"; break;
-    case SEEK_FRAME: sm = "FRAME"; break;
+    case SEEK_TRACK_NR:
+        sm = "TRACK_NR";
+        break;
+    case SEEK_ABS_TIME:
+        sm = "ABS_TIME";
+        value = upnpduration(target*1000);
+        break;
+    case SEEK_REL_TIME:
+        sm = "REL_TIME";
+        value = upnpduration(target*1000);
+        break;
+    case SEEK_ABS_COUNT:
+        sm = "ABS_COUNT";
+        break;
+    case SEEK_REL_COUNT:
+        sm = "REL_COUNT";
+        break;
+    case SEEK_CHANNEL_FREQ:
+        sm = "CHANNEL_FREQ";
+        break;
+    case SEEK_TAPE_INDEX:
+        sm = "TAPE-INDEX";
+        break;
+    case SEEK_FRAME:
+        sm = "FRAME";
+        break;
     default:
         return UPNP_E_INVALID_PARAM;
     }
 
     SoapOutgoing args(getServiceType(), "Seek");
     args("InstanceID", SoapHelp::i2s(instanceID))
-        ("Unit", sm)
-        ("Target", value);
+    ("Unit", sm)
+    ("Target", value);
     SoapIncoming data;
     return runAction(args, data);
 }
