@@ -1,48 +1,45 @@
-/* Copyright (C) 2013 J.F.Dockes
- *	 This program is free software; you can redistribute it and/or modify
- *	 it under the terms of the GNU General Public License as published by
- *	 the Free Software Foundation; either version 2 of the License, or
- *	 (at your option) any later version.
+/* Copyright (C) 2006-2016 J.F.Dockes
  *
- *	 This program is distributed in the hope that it will be useful,
- *	 but WITHOUT ANY WARRANTY; without even the implied warranty of
- *	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *	 GNU General Public License for more details.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *	 You should have received a copy of the GNU General Public License
- *	 along with this program; if not, write to the
- *	 Free Software Foundation, Inc.,
- *	 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ *   02110-1301 USA
  */
 #include "libupnpp/config.h"
 
 #include "libupnpp/control/cdirectory.hxx"
 
 #include <sys/types.h>
-#ifndef WIN32
-#include <regex.h>
-#else
-#include <regex>
-#endif
 
-#include <upnp/upnp.h>                  // for UPNP_E_SUCCESS, etc
-#include <upnp/upnptools.h>             // for UpnpGetErrorMessage
+#include <upnp.h>
+#include <upnptools.h>
 
-#include <functional>                   // for _Bind, bind, _1, _2
-#include <iostream>                     // for operator<<, basic_ostream, etc
-#include <set>                          // for set
-#include <string>                       // for string, operator<<, etc
-#include <vector>                       // for vector
+#include <functional>
+#include <iostream>
+#include <set>
+#include <string>
+#include <vector>
 
-#include "libupnpp/control/cdircontent.hxx"  // for UPnPDirContent
-#include "libupnpp/control/description.hxx"  // for UPnPDeviceDesc, etc
-#include "libupnpp/control/discovery.hxx"  // for UPnPDeviceDirectory, etc
-#include "libupnpp/log.hxx"             // for LOGDEB, LOGINF, LOGERR
-#include "libupnpp/soaphelp.hxx"        // for SoapOutgoing, SoapOutgoing, etc
-#include "libupnpp/upnpp_p.hxx"         // for csvToStrings
+#include "libupnpp/control/cdircontent.hxx"
+#include "libupnpp/control/description.hxx"
+#include "libupnpp/control/discovery.hxx"
+#include "libupnpp/log.hxx"
+#include "libupnpp/smallut.h"
+#include "libupnpp/soaphelp.hxx"
+#include "libupnpp/upnpp_p.hxx"
 
 using namespace std;
-using namespace STD_PLACEHOLDERS;
+using namespace std::placeholders;
 using namespace UPnPP;
 
 namespace UPnPClient {
@@ -50,59 +47,43 @@ namespace UPnPClient {
 // The service type string for Content Directories:
 const string ContentDirectory::SType("urn:schemas-upnp-org:service:ContentDirectory:1");
 
-#ifndef WIN32
-class SimpleRegexp {
-public:
-    SimpleRegexp(const string& exp, int flags) : m_ok(false) {
-        if (regcomp(&m_expr, exp.c_str(), flags) == 0) {
-            m_ok = true;
+// We don't include a version in comparisons, as we are satisfied with
+// version 1
+bool ContentDirectory::isCDService(const string& st)
+{
+    const string::size_type sz(SType.size()-2);
+    return !SType.compare(0, sz, st, 0, sz);
+}
+bool ContentDirectory::serviceTypeMatch(const std::string& tp)
+{
+    return isCDService(tp);
+}
+
+void ContentDirectory::evtCallback(
+    const std::unordered_map<string, string>& props)
+{
+    for (std::unordered_map<std::string, std::string>::const_iterator it =
+             props.begin(); it != props.end(); it++) {
+        if (!getReporter()) {
+            LOGDEB1("ContentDirectory::evtCallback: " << it->first << " -> "
+                    << it->second << endl);
+            continue;
         }
-    }
-    ~SimpleRegexp() {
-        regfree(&m_expr);
-    }
-    bool simpleMatch(const string& val) const {
-        if (!ok())
-            return false;
-        if (regexec(&m_expr, val.c_str(), 0, 0, 0) == 0) {
-            return true;
+        if (!it->first.compare("SystemUpdateID")) {
+            getReporter()->changed(it->first.c_str(), atoi(it->second.c_str()));
+
+        } else if (!it->first.compare("ContainerUpdateIDs") ||
+                   !it->first.compare("TransferIDs")) {
+            getReporter()->changed(it->first.c_str(),
+                                   it->second.c_str());
+
         } else {
-            return false;
+            LOGERR("ContentDirectory event: unknown variable: name [" <<
+                   it->first << "] value [" << it->second << endl);
+            getReporter()->changed(it->first.c_str(), it->second.c_str());
         }
     }
-    bool operator() (const string& val) const {
-        return simpleMatch(val);
-    }
-
-    bool ok() const {return m_ok;}
-private:
-    bool m_ok;
-    regex_t m_expr;
-};
-#else
-class SimpleRegexp {
-public:
-	SimpleRegexp(const string& exp, int flags)
-		: m_expr(exp, basic_regex<char>::flag_type(flags)), m_ok(true) {}
-	
-	bool simpleMatch(const string& val) const {
-		if (!ok())
-			return false;
-		return regex_match(val, m_expr);
-	}
-	bool operator() (const string& val) const {
-		return simpleMatch(val);
-	}
-
-	bool ok() const { return m_ok; }
-private:
-	basic_regex<char> m_expr;
-	bool m_ok;
-};
-#define REG_ICASE std::regex_constants::icase
-#define REG_NOSUB std::regex_constants::nosubs
-
-#endif
+}
 
 /*
   manufacturer: Bubblesoft model BubbleUPnP Media Server
@@ -111,19 +92,25 @@ private:
   manufacturer: PacketVideo model TwonkyMedia Server
   manufacturer: ? model MediaTomb
 */
-static const SimpleRegexp bubble_rx("bubble", REG_ICASE|REG_NOSUB);
-static const SimpleRegexp mediatomb_rx("mediatomb", REG_ICASE|REG_NOSUB);
-static const SimpleRegexp minidlna_rx("minidlna", REG_ICASE|REG_NOSUB);
-static const SimpleRegexp minim_rx("minim", REG_ICASE|REG_NOSUB);
-static const SimpleRegexp twonky_rx("twonky", REG_ICASE|REG_NOSUB);
+static const int sreflags = SimpleRegexp::SRE_ICASE|SimpleRegexp::SRE_NOSUB;
+static const SimpleRegexp bubble_rx("bubble", sreflags);
+static const SimpleRegexp mediatomb_rx("mediatomb", sreflags);
+static const SimpleRegexp minidlna_rx("minidlna", sreflags);
+static const SimpleRegexp minim_rx("minim", sreflags);
+static const SimpleRegexp twonky_rx("twonky", sreflags);
 
 ContentDirectory::ContentDirectory(const UPnPDeviceDesc& device,
                                    const UPnPServiceDesc& service)
-    : Service(device, service), m_rdreqcnt(200), m_serviceKind(CDSKIND_UNKNOWN)
+    : Service(device, service)
 {
-    LOGERR("ContentDirectory::ContentDirectory: manufacturer: " << 
-           getManufacturer() << " model " << getModelName() << endl);
+    LOGDEB("ContentDirectory::ContentDirectory: manufacturer: [" <<
+           getManufacturer() << "] model [" << getModelName() << "]\n");
+    serviceInit(device, service);
+}
 
+bool ContentDirectory::serviceInit(const UPnPDeviceDesc&,
+                                   const UPnPServiceDesc&)
+{
     if (bubble_rx(getModelName())) {
         m_serviceKind = CDSKIND_BUBBLE;
         LOGDEB1("ContentDirectory::ContentDirectory: BUBBLE" << endl);
@@ -142,24 +129,12 @@ ContentDirectory::ContentDirectory(const UPnPDeviceDesc& device,
     } else if (twonky_rx(getModelName())) {
         m_serviceKind = CDSKIND_TWONKY;
         LOGDEB1("ContentDirectory::ContentDirectory: TWONKY" << endl);
-    } 
-    registerCallback();
-}
-ContentDirectory::~ContentDirectory()
-{
-    unregisterCallback();
-}
-
-// We don't include a version in comparisons, as we are satisfied with
-// version 1
-bool ContentDirectory::isCDService(const string& st)
-{
-    const string::size_type sz(SType.size()-2);
-    return !SType.compare(0, sz, st, 0, sz);
+    }
+    return true;
 }
 
 static bool DSAccum(vector<CDSH>* out,
-                    const UPnPDeviceDesc& device, 
+                    const UPnPDeviceDesc& device,
                     const UPnPServiceDesc& service)
 {
     if (ContentDirectory::isCDService(service.serviceType)) {
@@ -176,7 +151,7 @@ bool ContentDirectory::getServices(vector<CDSH>& vds)
     return !vds.empty();
 }
 
-// Get server by friendly name. 
+// Get server by friendly name.
 bool ContentDirectory::getServerByName(const string& fname, CDSH& server)
 {
     UPnPDeviceDesc ddesc;
@@ -185,8 +160,8 @@ bool ContentDirectory::getServerByName(const string& fname, CDSH& server)
         return false;
 
     found = false;
-    for (std::vector<UPnPServiceDesc>::const_iterator it = 
-             ddesc.services.begin(); it != ddesc.services.end(); it++) {
+    for (std::vector<UPnPServiceDesc>::const_iterator it =
+                ddesc.services.begin(); it != ddesc.services.end(); it++) {
         if (isCDService(it->serviceType)) {
             server = CDSH(new ContentDirectory(ddesc, *it));
             found = true;
@@ -196,53 +171,27 @@ bool ContentDirectory::getServerByName(const string& fname, CDSH& server)
     return found;
 }
 
-#if 0
-static int asyncReaddirCB(Upnp_EventType et, void *vev, void *cookie)
-{
-    LOGDEB("asyncReaddirCB: " << LibUPnP::evTypeAsString(et) << endl);
-    struct Upnp_Action_Complete *act = (struct Upnp_Action_Complete*)vev;
-
-    LOGDEB("asyncReaddirCB: errcode " << act->ErrCode << 
-           " cturl " <<  UpnpString_get_String(act->CtrlUrl) << 
-           " actionrequest " << endl << 
-           ixmlwPrintDoc(act->ActionRequest) << endl <<
-           " actionresult " << ixmlwPrintDoc(act->ActionResult) << endl);
-    return 0;
-}
-    int ret = 
-        UpnpSendActionAsync(hdl, getActionURL().c_str(), getServiceType().c_str(),
-        0 /*devUDN*/, request, asyncReaddirCB, 0);
-    sleep(10);
-    return -1;
-#endif
-
-void ContentDirectory::evtCallback(const STD_UNORDERED_MAP<string, string>&)
-{
-}
-
 void ContentDirectory::registerCallback()
 {
-    LOGDEB("ContentDirectory::registerCallback"<< endl);
-    Service::registerCallback(bind(&ContentDirectory::evtCallback, 
-                                   this, _1));
+    Service::registerCallback(bind(&ContentDirectory::evtCallback, this, _1));
 }
 
 int ContentDirectory::readDirSlice(const string& objectId, int offset,
-                                          int count, UPnPDirContent& dirbuf,
-                                          int *didread, int *total)
+                                   int count, UPnPDirContent& dirbuf,
+                                   int *didread, int *total)
 {
-    LOGDEB("CDService::readDirSlice: objId [" << objectId << "] offset " << 
+    LOGDEB("CDService::readDirSlice: objId [" << objectId << "] offset " <<
            offset << " count " << count << endl);
 
     // Create request
     // Some devices require an empty SortCriteria, else bad params
     SoapOutgoing args(getServiceType(), "Browse");
     args("ObjectID", objectId)
-        ("BrowseFlag", "BrowseDirectChildren")
-        ("Filter", "*")
-        ("SortCriteria", "")
-        ("StartingIndex", SoapHelp::i2s(offset))
-        ("RequestedCount", SoapHelp::i2s(count));
+    ("BrowseFlag", "BrowseDirectChildren")
+    ("Filter", "*")
+    ("SortCriteria", "")
+    ("StartingIndex", SoapHelp::i2s(offset))
+    ("RequestedCount", SoapHelp::i2s(count));
 
     SoapIncoming data;
     int ret = runAction(args, data);
@@ -252,8 +201,8 @@ int ContentDirectory::readDirSlice(const string& objectId, int offset,
 
     string tbuf;
     if (!data.get("NumberReturned", didread) ||
-        !data.get("TotalMatches", total) ||
-        !data.get("Result", &tbuf)) {
+            !data.get("TotalMatches", total) ||
+            !data.get("Result", &tbuf)) {
         LOGERR("CDService::readDir: missing elts in response" << endl);
         return UPNP_E_BAD_RESPONSE;
     }
@@ -263,12 +212,9 @@ int ContentDirectory::readDirSlice(const string& objectId, int offset,
         return UPNP_E_BAD_RESPONSE;
     }
 
-#if 0
-    cerr << "CDService::readDirSlice: count " << count <<
-        " offset " << offset <<
-        " total " << *total << endl;
-    cerr << " result " << tbuf << endl;
-#endif
+    LOGDEB0("ContentDirectory::readDirSlice: got count " << count <<
+            " offset " << offset << " total " << *total << "Data:\n" <<
+            tbuf << "\n");
 
     dirbuf.parse(tbuf);
 
@@ -276,7 +222,7 @@ int ContentDirectory::readDirSlice(const string& objectId, int offset,
 }
 
 int ContentDirectory::readDir(const string& objectId,
-                                     UPnPDirContent& dirbuf)
+                              UPnPDirContent& dirbuf)
 {
     LOGDEB("CDService::readDir: url [" << getActionURL() << "] type [" <<
            getServiceType() << "] udn [" << getDeviceId() << "] objId [" <<
@@ -303,17 +249,17 @@ int ContentDirectory::searchSlice(const string& objectId,
                                   int offset, int count, UPnPDirContent& dirbuf,
                                   int *didread, int *total)
 {
-    LOGDEB("CDService::searchSlice: objId [" << objectId << "] offset " << 
+    LOGDEB("CDService::searchSlice: objId [" << objectId << "] offset " <<
            offset << " count " << count << endl);
 
     // Create request
     SoapOutgoing args(getServiceType(), "Search");
     args("ContainerID", objectId)
-        ("SearchCriteria", ss)
-        ("Filter", "*")
-        ("SortCriteria", "")
-        ("StartingIndex", SoapHelp::i2s(offset))
-        ("RequestedCount", SoapHelp::i2s(count)); 
+    ("SearchCriteria", ss)
+    ("Filter", "*")
+    ("SortCriteria", "")
+    ("StartingIndex", SoapHelp::i2s(offset))
+    ("RequestedCount", SoapHelp::i2s(count));
 
     SoapIncoming data;
     int ret = runAction(args, data);
@@ -326,8 +272,8 @@ int ContentDirectory::searchSlice(const string& objectId,
 
     string tbuf;
     if (!data.get("NumberReturned", didread) ||
-        !data.get("TotalMatches", total) ||
-        !data.get("Result", &tbuf)) {
+            !data.get("TotalMatches", total) ||
+            !data.get("Result", &tbuf)) {
         LOGERR("CDService::search: missing elts in response" << endl);
         return UPNP_E_BAD_RESPONSE;
     }
@@ -345,8 +291,8 @@ int ContentDirectory::search(const string& objectId,
                              const string& ss,
                              UPnPDirContent& dirbuf)
 {
-    LOGDEB("CDService::search: url [" << getActionURL() << "] type [" << 
-           getServiceType() << "] udn [" << getDeviceId() << "] objid [" << 
+    LOGDEB("CDService::search: url [" << getActionURL() << "] type [" <<
+           getServiceType() << "] udn [" << getDeviceId() << "] objid [" <<
            objectId <<  "] search [" << ss << "]" << endl);
 
     int offset = 0;
@@ -373,7 +319,7 @@ int ContentDirectory::getSearchCapabilities(set<string>& result)
     SoapIncoming data;
     int ret = runAction(args, data);
     if (ret != UPNP_E_SUCCESS) {
-        LOGINF("CDService::getSearchCapa: UpnpSendAction failed: " << 
+        LOGINF("CDService::getSearchCapa: UpnpSendAction failed: " <<
                UpnpGetErrorMessage(ret) << endl);
         return ret;
     }
@@ -397,7 +343,7 @@ int ContentDirectory::getSearchCapabilities(set<string>& result)
 }
 
 int ContentDirectory::getMetadata(const string& objectId,
-                                         UPnPDirContent& dirbuf)
+                                  UPnPDirContent& dirbuf)
 {
     LOGDEB("CDService::getMetadata: url [" << getActionURL() << "] type [" <<
            getServiceType() << "] udn [" << getDeviceId() << "] objId [" <<
@@ -406,14 +352,14 @@ int ContentDirectory::getMetadata(const string& objectId,
     SoapOutgoing args(getServiceType(), "Browse");
     SoapIncoming data;
     args("ObjectID", objectId)
-        ("BrowseFlag", "BrowseMetadata")
-        ("Filter", "*")
-        ("SortCriteria", "")
-        ("StartingIndex", "0")
-        ("RequestedCount", "1");
+    ("BrowseFlag", "BrowseMetadata")
+    ("Filter", "*")
+    ("SortCriteria", "")
+    ("StartingIndex", "0")
+    ("RequestedCount", "1");
     int ret = runAction(args, data);
     if (ret != UPNP_E_SUCCESS) {
-        LOGINF("CDService::getmetadata: UpnpSendAction failed: " << 
+        LOGINF("CDService::getmetadata: UpnpSendAction failed: " <<
                UpnpGetErrorMessage(ret) << endl);
         return ret;
     }
